<?php
/**
 * Plugin Name: Autoload Options Manager
 * Description: Displays autoloaded options in a table, allows safe deletion of non-critical options, and protects essential WordPress options.
 * Version: 2.4
 * Author: MBSMGROUP
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

// Register admin menu
function aot_add_admin_menu()
{
    add_menu_page(
        'Autoloaded Options',
        'Autoloaded Options',
        'manage_options',
        'autoloaded-options',
        'aot_display_autoloaded_options',
        'dashicons-admin-generic',
        100
    );
}
add_action('admin_menu', 'aot_add_admin_menu');

// List of essential WordPress options to exclude (never delete these)
function aot_get_protected_options() {
    return [
        'siteurl', 'home', 'blogname', 'blogdescription', 'admin_email',
        'start_of_week', 'posts_per_page', 'date_format', 'time_format',
        'timezone_string', 'default_role', 'permalink_structure',
        'active_plugins', 'template', 'stylesheet', 'current_theme'
    ];
}

// Function to determine the source of an option
function aot_get_option_source($option_name) {
    $plugin_prefixes = [
        'woocommerce_' => 'WooCommerce',
        'yoast_' => 'Yoast SEO',
        'jetpack_' => 'Jetpack',
        'akismet_' => 'Akismet'
    ];

    foreach ($plugin_prefixes as $prefix => $plugin_name) {
        if (strpos($option_name, $prefix) === 0) {
            return $plugin_name;
        }
    }

    if (strpos($option_name, '_transient_') === 0 || strpos($option_name, '_site_transient_') === 0) {
        return 'WordPress Core';
    }

    return 'Custom Plugin/Theme';
}

// Function to check if an option is critical (related to active theme/plugins)
function aot_is_option_critical($option_name) {
    $active_theme = wp_get_theme();
    $active_theme_prefix = 'theme_mods_' . $active_theme->get_stylesheet();

    if (strpos($option_name, $active_theme_prefix) === 0) {
        return true;
    }

    $active_plugins = get_option('active_plugins');
    foreach ($active_plugins as $plugin) {
        $plugin_slug = dirname($plugin);
        if (strpos($option_name, $plugin_slug) === 0) {
            return true;
        }
    }

    return false;
}

// Function to get plugin/theme name, status, and link
function aot_get_plugin_theme_info($option_name) {
    $all_plugins = get_plugins();
    $active_plugins = get_option('active_plugins');

    foreach ($all_plugins as $plugin_path => $plugin_data) {
        $plugin_slug = dirname($plugin_path);
        if (strpos($option_name, $plugin_slug) === 0) {
            $status = in_array($plugin_path, $active_plugins) ? 'Installed & Active' : 'Installed & Inactive';
            $plugin_link = admin_url('plugins.php?plugin_status=all&paged=1&s=' . $plugin_data['Name']);
            return [
                'name' => $plugin_data['Name'],
                'status' => $status,
                'type' => 'Plugin',
                'link' => $plugin_link
            ];
        }
    }

    $all_themes = wp_get_themes();
    $active_theme = wp_get_theme();

    foreach ($all_themes as $theme_slug => $theme_data) {
        $theme_prefix = 'theme_mods_' . $theme_slug;
        if (strpos($option_name, $theme_prefix) === 0) {
            $status = ($theme_slug === $active_theme->get_stylesheet()) ? 'Installed & Active' : 'Installed & Inactive';
            $theme_link = admin_url('themes.php');
            return [
                'name' => $theme_data->get('Name'),
                'status' => $status,
                'type' => 'Theme',
                'link' => $theme_link
            ];
        }
    }

    return [
        'name' => 'N/A',
        'status' => 'N/A',
        'type' => 'N/A',
        'link' => '#'
    ];
}

// Display the autoloaded options in a table with delete functionality
function aot_display_autoloaded_options()
{
    global $wpdb;

    if (!current_user_can('manage_options')) {
        wp_die(__('You do not have sufficient permissions to access this page.'));
    }

    if (isset($_POST['delete_options']) && !empty($_POST['option_names'])) {
        $deleted_options = [];
        $protected_options = aot_get_protected_options();
        $option_names = $_POST['option_names'];

        foreach ($option_names as $option_name) {
            if (in_array($option_name, $protected_options)) {
                continue;
            }

            if (delete_option($option_name)) {
                $deleted_options[] = $option_name;
            }
        }

        if (!empty($deleted_options)) {
            echo '<div class="notice notice-success is-dismissible"><p>Selected options deleted successfully: ' . esc_html(implode(', ', $deleted_options)) . '</p></div>';
        }
    }

    $search_query = isset($_GET['s']) ? sanitize_text_field($_GET['s']) : '';
    $search_condition = $search_query ? "AND option_name LIKE '%" . esc_sql($search_query) . "%'" : '';

    $protected_options = aot_get_protected_options();
    $protected_options_placeholders = implode(',', array_fill(0, count($protected_options), '%s'));

    $results = $wpdb->get_results(
        $wpdb->prepare(
            "SELECT option_name
             FROM $wpdb->options
             WHERE autoload = 'yes'
             AND option_name NOT IN ($protected_options_placeholders)
             $search_condition",
            ...$protected_options
        )
    );

    echo '<div class="wrap">';
    echo '<h1>Autoloaded Options</h1>';
    echo '<p><strong>Note:</strong> Only non-critical options are displayed. Be cautious when deleting options.</p>';

    echo '<form method="get" action="">';
    echo '<input type="hidden" name="page" value="autoloaded-options">';
    echo '<p class="search-box">';
    echo '<label class="screen-reader-text" for="search-options">Search Options:</label>';
    echo '<input type="search" id="search-options" name="s" value="' . esc_attr($search_query) . '">';
    echo '<input type="submit" id="search-submit" class="button" value="Search Options">';
    echo '</p>';
    echo '</form>';

    echo '<form method="post" action="">';
    echo '<table class="wp-list-table widefat fixed striped">';
    echo '<thead><tr><th><input type="checkbox" id="select_all"></th><th>Option Name</th><th>Source</th><th>Status</th><th>Plugin/Theme Status</th></tr></thead>';
    echo '<tbody>';

    if (!empty($results)) {
        foreach ($results as $row) {
            $source = aot_get_option_source($row->option_name);
            $is_critical = aot_is_option_critical($row->option_name);
            $status = $is_critical ? 'Critical' : 'Safe to Delete';
            $row_class = $is_critical ? 'critical' : ($source === 'Custom Plugin/Theme' ? 'warning' : 'safe');

            $plugin_theme_info = aot_get_plugin_theme_info($row->option_name);
            $plugin_theme_name = $plugin_theme_info['name'];
            $plugin_theme_status = $plugin_theme_info['status'];
            $plugin_theme_link = $plugin_theme_info['link'];

            echo '<tr class="' . esc_attr($row_class) . '">';
            echo '<td><input type="checkbox" name="option_names[]" value="' . esc_attr($row->option_name) . '"></td>';
            echo '<td>' . esc_html($row->option_name) . '</td>';
            echo '<td>' . esc_html($source) . '</td>';
            echo '<td>' . esc_html($status) . '</td>';
            echo '<td>';
            if ($plugin_theme_name !== 'N/A') {
                echo '<a href="' . esc_url($plugin_theme_link) . '" target="_blank">' . esc_html($plugin_theme_name) . '</a> (' . esc_html($plugin_theme_status) . ')';
            } else {
                echo 'N/A';
            }
            echo '</td>';
            echo '</tr>';
        }
    } else {
        echo '<tr><td colspan="5">No autoloaded options found.</td></tr>';
    }

    echo '</tbody></table>';
    echo '<br>';
    echo '<input type="submit" name="delete_options" class="button button-primary" value="Delete Selected Options" onclick="return confirm(\'Are you sure you want to delete the selected options? This action cannot be undone.\');">';
    echo '</form>';
    echo '</div>';

    echo '<script type="text/javascript">
        document.getElementById("select_all").addEventListener("click", function(event){
            var checkboxes = document.querySelectorAll(\'input[type="checkbox"]\');
            for (var i = 0; i < checkboxes.length; i++) {
                checkboxes[i].checked = event.target.checked;
            }
        });
    </script>';

    echo '<style>
        .critical {
            background-color: #ffebee;
        }
        .warning {
            background-color: #fff3e0;
        }
        .safe {
            background-color: #e8f5e9;
        }
    </style>';
}
